# frozen_string_literal: true

module Hermes
  # Threshold clas
  class Threshold
    include Hermes::Concerns::CommonFieldsConcern
    field :type, type: String
    field :codes, type: Array
    field :thresholds_map, type: Hash

    has_many :housing_offers, class_name: 'Hermes::HousingOffer'

    writer_roles 'Superadmin', 'Administrator'
    reader_roles 'Manager'
    owner_field :user_id

    after_create :after_create_actions
    after_update :after_actions

    validates_presence_of :type, :thresholds_map
    #TODO : handle string data for thresholds_map
    def subtype(subtype_name)
      thresholds_map[subtype_name]
    end

    def thresholds_map=(h)
      super(
        if h.is_a? Hash
          h.all_to(Integer)
        else
          {}
        end
      )
    end

    def thresholds(subtype_name, type_name)
      sub_type = subtype(subtype_name)
      return unless sub_type

      field_name = case type_name
                   when 'ceils'
                     'thresholds'
                   else
                     type_name
                   end
      sub_type["#{field_name}_list"]
    end

    def threshold(subtype_name, type_name, index)
      thresholds = thresholds(subtype_name, type_name)
      return unless thresholds

      thresholds[index] || 0.0
    end

    def threshold_addition(subtype_name, type_name, index)
      sub_type = subtype(subtype_name)
      thresholds = thresholds(subtype_name, type_name)
      return unless thresholds

      addition_value = sub_type['another_person']
      return unless addition_value

      thresholds.last + (addition_value * index) || 0.0
    end

    def udpate_housing_offers
      self.housing_offers = Hermes::HousingOffer.where(threshold_type: type)
    end

    def set_offer_thresholds(offer)
      offer_subtype = offer.threshold_subtype
      return unless offer_subtype

      %w[ceils floors].each do |field_type|
        (1..6).each do |i|
          offer.send(
            "#{field_type.singularize}_category_#{i}=",
            threshold(offer_subtype, field_type, i - 1) || 0.0
          )
        end
        (7..12).each do |i|
          j = i - 6
          offer.send(
            "#{field_type.singularize}_category_#{i}=",
            threshold_addition(offer_subtype, field_type, j) || 0.0
          )
        end
      end
    end

    def update_related_offers
      housing_offers.each do |offer|
        set_offer_thresholds(offer)
        offer.save
      end
    end

    def after_create_actions
      udpate_housing_offers
      update_related_offers
    end

    def after_actions
      update_related_offers if thresholds_map_changed?
    end
  end
end


# mapping = {
#   'PLA I'=> ['PLN'],
#   'PLA LM'=> ['PLM'],
#   'PLS 7/92 à 8/93'=> ['PS2'],
#   'PLI 3/96 à 2/01'=> ['PL3'],
#   'PLA'=> ['PLA'],
#   'PLS 1/97 (sans dérogation)'=> ['PS1'],
#   'PLI AVANT 7/92'=> ['PL1'],
#   'PLI 8/93 à 2/96'=> ['PL2'],
#   'PLI 8/2004 à 12/2014'=> ['PL5'],
#   'PLI 2015'=> ['PL6'],
#   'PLA TS'=> ['PTS'],
#   'HLM'=> ['HLM', 'HLO'],
#   'PLUS'=> ['PLU'],
#   'PLA CCF (sans dérogation)'=> ['CFF'],
#   'LLI'=> ['LLI'],
#   'ILM (SA HLM)'=> ['IL1'],
#   'PINEL'=> ['PIN'],
#   'BESSON NEUF'=> ['BNE', 'BOR'],
#   'BESSON ANCIEN'=> ['BAN'],
# }

# mapping.each do |k, c|
#   t = Hermes::Threshold.find_by(type: k)
#   # puts t.type
#   t.codes = c
#   t.save
# end
